@@ -1,8 +1,6 @@
 # --
 # Kernel/Modules/AgentTicketActionCommon.pm - common file for several modules
-# Copyright (C) 2001-2014 OTRS AG, http://otrs.com/
-# --
-# $Id: AgentTicketActionCommon.pm,v 1.81.2.10 2013-01-30 00:00:06 cr Exp $
+# Copyright (C) 2001-2013 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -114,7 +112,7 @@ sub Run {
     );
     my %AclAction = $Self->{TicketObject}->TicketAclActionData();
 
-    # check if ACL resctictions if exist
+    # check if ACL restrictions exist
     if ( IsHashRefWithData( \%AclAction ) ) {
 
         # show error screen if ACL prohibits this action
@@ -210,7 +208,7 @@ sub Run {
     my %GetParam;
     for my $Key (
         qw(
-        NewStateID NewPriorityID TimeUnits ArticleTypeID Title Body Subject
+        NewStateID NewPriorityID TimeUnits ArticleTypeID Title Body Subject NewQueueID
         Year Month Day Hour Minute NewOwnerID NewOwnerType OldOwnerID NewResponsibleID
         TypeID ServiceID SLAID Expand
         )
@@ -239,7 +237,7 @@ sub Run {
     # convert dynamic field values into a structure for ACLs
     my %DynamicFieldACLParameters;
     DYNAMICFIELD:
-    for my $DynamicField ( keys %DynamicFieldValues ) {
+    for my $DynamicField ( sort keys %DynamicFieldValues ) {
         next DYNAMICFIELD if !$DynamicField;
         next DYNAMICFIELD if !$DynamicFieldValues{$DynamicField};
 
@@ -428,6 +426,10 @@ sub Run {
             # check if field has PossibleValues property in its configuration
             if ( IsHashRefWithData( $DynamicFieldConfig->{Config}->{PossibleValues} ) ) {
 
+                # convert possible values key => value to key => key for ACLs usign a Hash slice
+                my %AclData = %{ $DynamicFieldConfig->{Config}->{PossibleValues} };
+                @AclData{ keys %AclData } = keys %AclData;
+
                 # set possible values filter from ACLs
                 my $ACL = $Self->{TicketObject}->TicketAcl(
                     %GetParam,
@@ -435,12 +437,16 @@ sub Run {
                     TicketID      => $Self->{TicketID},
                     ReturnType    => 'Ticket',
                     ReturnSubType => 'DynamicField_' . $DynamicFieldConfig->{Name},
-                    Data          => $DynamicFieldConfig->{Config}->{PossibleValues},
+                    Data          => \%AclData,
                     UserID        => $Self->{UserID},
                 );
                 if ($ACL) {
                     my %Filter = $Self->{TicketObject}->TicketAclData();
-                    $PossibleValuesFilter = \%Filter;
+
+                    # convert Filer key => key back to key => value using map
+                    %{$PossibleValuesFilter}
+                        = map { $_ => $DynamicFieldConfig->{Config}->{PossibleValues}->{$_} }
+                        keys %Filter;
                 }
             }
 
@@ -617,9 +623,45 @@ sub Run {
             }
         }
 
+        # move ticket to a new queue, but only if the queue was changed
+        if (
+            $Self->{Config}->{Queue}
+            && $GetParam{NewQueueID}
+            && $GetParam{NewQueueID} ne $Ticket{QueueID}
+            )
+        {
+
+            # move ticket (send notification if no new owner is selected)
+            my $BodyAsText = '';
+            if ( $Self->{LayoutObject}->{BrowserRichText} ) {
+                $BodyAsText = $Self->{LayoutObject}->RichText2Ascii(
+                    String => $GetParam{Body} || 0,
+                );
+            }
+            else {
+                $BodyAsText = $GetParam{Body} || 0;
+            }
+            my $Move = $Self->{TicketObject}->TicketQueueSet(
+                QueueID            => $GetParam{NewQueueID},
+                UserID             => $Self->{UserID},
+                TicketID           => $Self->{TicketID},
+                SendNoNotification => $GetParam{NewUserID},
+                Comment            => $BodyAsText,
+            );
+            if ( !$Move ) {
+                return $Self->{LayoutObject}->ErrorScreen();
+            }
+        }
+
         # add note
         my $ArticleID = '';
         if ( $Self->{Config}->{Note} ) {
+
+            # if there is no ArticleTypeID, use the default value
+            if ( !defined $GetParam{ArticleTypeID} ) {
+                $GetParam{ArticleType} = $Self->{Config}->{ArticleTypeDefault};
+            }
+
             my $MimeType = 'text/plain';
             if ( $Self->{LayoutObject}->{BrowserRichText} ) {
                 $MimeType = 'text/html';
@@ -701,12 +743,12 @@ sub Run {
                 );
             }
 
-            # remove pre submited attachments
+            # remove pre submitted attachments
             $Self->{UploadCacheObject}->FormIDRemove( FormID => $Self->{FormID} );
         }
 
         # set dynamic fields
-        # cycle trough the activated Dynamic Fields for this screen
+        # cycle through the activated Dynamic Fields for this screen
         DYNAMICFIELD:
         for my $DynamicFieldConfig ( @{ $Self->{DynamicField} } ) {
             next DYNAMICFIELD if !IsHashRefWithData($DynamicFieldConfig);
@@ -755,7 +797,7 @@ sub Run {
                 );
             }
 
-            # set pending time on pendig state
+            # set pending time on pending state
             elsif ( $StateData{TypeName} =~ /^pending/i ) {
 
                 # set pending time
@@ -795,12 +837,12 @@ sub Run {
             $ServiceID = $Ticket{ServiceID} || '';
         }
 
-        my $QueueID = $Ticket{QueueID};
+        my $QueueID = $GetParam{NewQueueID} || $Ticket{QueueID};
 
         # convert dynamic field values into a structure for ACLs
         my %DynamicFieldACLParameters;
         DYNAMICFIELD:
-        for my $DynamicField ( keys %DynamicFieldValues ) {
+        for my $DynamicField ( sort keys %DynamicFieldValues ) {
             next DYNAMICFIELD if !$DynamicField;
             next DYNAMICFIELD if !$DynamicFieldValues{$DynamicField};
 
@@ -839,7 +881,7 @@ sub Run {
         );
 
         # reset previous ServiceID to reset SLA-List if no service is selected
-        if ( !$Services->{$ServiceID} ) {
+        if ( !defined $ServiceID || !$Services->{$ServiceID} ) {
             $ServiceID = '';
         }
         my $SLAs = $Self->_GetSLAs(
@@ -851,10 +893,10 @@ sub Run {
         my $NextStates = $Self->_GetNextStates(
             %GetParam,
             CustomerUserID => $CustomerUser || '',
-            QueueID        => $QueueID      || 1,
+            QueueID => $QueueID,
         );
 
-        # update Dynamc Fields Possible Values via AJAX
+        # update Dynamic Fields Possible Values via AJAX
         my @DynamicFieldAJAX;
 
         # cycle trough the activated Dynamic Fields for this screen
@@ -871,20 +913,25 @@ sub Run {
                 DynamicFieldConfig => $DynamicFieldConfig,
             );
 
+            # convert possible values key => value to key => key for ACLs using a Hash slice
+            my %AclData = %{$PossibleValues};
+            @AclData{ keys %AclData } = keys %AclData;
+
             # set possible values filter from ACLs
             my $ACL = $Self->{TicketObject}->TicketAcl(
                 %GetParam,
                 Action        => $Self->{Action},
-                TicketID      => $Self->{TicketID},
-                QueueID       => $QueueID || 0,
+                QueueID       => $QueueID,
                 ReturnType    => 'Ticket',
                 ReturnSubType => 'DynamicField_' . $DynamicFieldConfig->{Name},
-                Data          => $PossibleValues,
+                Data          => \%AclData,
                 UserID        => $Self->{UserID},
             );
             if ($ACL) {
                 my %Filter = $Self->{TicketObject}->TicketAclData();
-                $PossibleValues = \%Filter;
+
+                # convert Filer key => key back to key => value using map
+                %{$PossibleValues} = map { $_ => $PossibleValues->{$_} } keys %Filter;
             }
 
             # add dynamic field to the list of fields to update
@@ -899,6 +946,7 @@ sub Run {
                 }
             );
         }
+
         my $JSON = $Self->{LayoutObject}->BuildSelectionJSON(
             [
 
@@ -1003,6 +1051,10 @@ sub Run {
             # check if field has PossibleValues property in its configuration
             if ( IsHashRefWithData( $DynamicFieldConfig->{Config}->{PossibleValues} ) ) {
 
+                # convert possible values key => value to key => key for ACLs usign a Hash slice
+                my %AclData = %{ $DynamicFieldConfig->{Config}->{PossibleValues} };
+                @AclData{ keys %AclData } = keys %AclData;
+
                 # set possible values filter from ACLs
                 my $ACL = $Self->{TicketObject}->TicketAcl(
                     %GetParam,
@@ -1010,12 +1062,16 @@ sub Run {
                     TicketID      => $Self->{TicketID},
                     ReturnType    => 'Ticket',
                     ReturnSubType => 'DynamicField_' . $DynamicFieldConfig->{Name},
-                    Data          => $DynamicFieldConfig->{Config}->{PossibleValues},
+                    Data          => \%AclData,
                     UserID        => $Self->{UserID},
                 );
                 if ($ACL) {
                     my %Filter = $Self->{TicketObject}->TicketAclData();
-                    $PossibleValuesFilter = \%Filter;
+
+                    # convert Filer key => key back to key => value using map
+                    %{$PossibleValuesFilter}
+                        = map { $_ => $DynamicFieldConfig->{Config}->{PossibleValues}->{$_} }
+                        keys %Filter;
                 }
             }
 
@@ -1023,7 +1079,7 @@ sub Run {
             my $Value;
 
             # only get values for Ticket fields (all screens based on AgentTickeActionCommon
-            # generates a new article, then article fields will be always empty at the beginign)
+            # generates a new article, then article fields will be always empty at the beginning)
             if ( $DynamicFieldConfig->{ObjectType} eq 'Ticket' ) {
 
                 # get value stored on the database from Ticket
@@ -1174,6 +1230,35 @@ sub _Mask {
             Data => {%Param},
         );
     }
+
+    if ( $Self->{Config}->{Queue} ) {
+
+        # fetch all queues
+        my %MoveQueues = $Self->{TicketObject}->TicketMoveList(
+            TicketID => $Self->{TicketID},
+            UserID   => $Self->{UserID},
+            Action   => $Self->{Action},
+            Type     => 'move_into',
+        );
+
+        # set move queues
+        $Param{QueuesStrg} = $Self->{LayoutObject}->AgentQueueListOption(
+            Data => { %MoveQueues, '' => '-' },
+            Multiple       => 0,
+            Size           => 0,
+            Class          => 'NewQueueID',
+            Name           => 'NewQueueID',
+            SelectedID     => $Param{NewQueueID},
+            CurrentQueueID => $Param{QueueID},
+            OnChangeSubmit => 0,
+        );
+
+        $Self->{LayoutObject}->Block(
+            Name => 'Queue',
+            Data => {%Param},
+        );
+    }
+
     if ( $Self->{Config}->{Owner} ) {
 
         # get user of own groups
@@ -1193,7 +1278,7 @@ sub _Mask {
                 Result  => 'HASH',
                 Cached  => 1,
             );
-            for my $UserID ( keys %MemberList ) {
+            for my $UserID ( sort keys %MemberList ) {
                 $ShownUsers{$UserID} = $AllGroupsMembers{$UserID};
             }
         }
@@ -1218,6 +1303,7 @@ sub _Mask {
                 $Counter++;
             }
         }
+
         my $OldOwnerSelectedID = '';
         if ( $Param{OldOwnerID} ) {
             $OldOwnerSelectedID = $Param{OldOwnerID};
@@ -1265,17 +1351,18 @@ sub _Mask {
                 Result  => 'HASH',
                 Cached  => 1,
             );
-            for my $UserID ( keys %MemberList ) {
+            for my $UserID ( sort keys %MemberList ) {
                 $ShownUsers{$UserID} = $AllGroupsMembers{$UserID};
             }
         }
 
         # get responsible
         $Param{ResponsibleStrg} = $Self->{LayoutObject}->BuildSelection(
-            Data       => \%ShownUsers,
-            SelectedID => $Param{NewResponsibleID} || $Ticket{ResponsibleID},
-            Name       => 'NewResponsibleID',
-            Size       => 1,
+            Data         => \%ShownUsers,
+            SelectedID   => $Param{NewResponsibleID},
+            Name         => 'NewResponsibleID',
+            PossibleNone => 1,
+            Size         => 1,
         );
         $Self->{LayoutObject}->Block(
             Name => 'Responsible',
@@ -1379,6 +1466,11 @@ sub _Mask {
 
         # add rich text editor
         if ( $Self->{LayoutObject}->{BrowserRichText} ) {
+
+            # use height/width defined for this screen
+            $Param{RichTextHeight} = $Self->{Config}->{RichTextHeight} || 0;
+            $Param{RichTextWidth}  = $Self->{Config}->{RichTextWidth}  || 0;
+
             $Self->{LayoutObject}->Block(
                 Name => 'RichText',
                 Data => \%Param,
@@ -1399,7 +1491,7 @@ sub _Mask {
                 Result  => 'HASH',
                 Cached  => 1,
             );
-            for my $UserID ( keys %MemberList ) {
+            for my $UserID ( sort keys %MemberList ) {
                 $ShownUsers{$UserID} = $AllGroupsMembers{$UserID};
             }
             my $InformAgentSize = $Self->{ConfigObject}->Get('Ticket::Frontend::InformAgentMaxSize')
@@ -1486,22 +1578,25 @@ sub _Mask {
         }
 
         # get possible notes
-        my %DefaultNoteTypes = %{ $Self->{Config}->{ArticleTypes} };
-        my %NoteTypes = $Self->{TicketObject}->ArticleTypeList( Result => 'HASH' );
-        for my $KeyNoteType ( keys %NoteTypes ) {
-            if ( !$DefaultNoteTypes{ $NoteTypes{$KeyNoteType} } ) {
-                delete $NoteTypes{$KeyNoteType};
+        if ( $Self->{Config}->{ArticleTypes} ) {
+            my %DefaultNoteTypes = %{ $Self->{Config}->{ArticleTypes} };
+            my %NoteTypes = $Self->{TicketObject}->ArticleTypeList( Result => 'HASH' );
+            for my $KeyNoteType ( sort keys %NoteTypes ) {
+                if ( !$DefaultNoteTypes{ $NoteTypes{$KeyNoteType} } ) {
+                    delete $NoteTypes{$KeyNoteType};
+                }
             }
+
+            $Param{ArticleTypeStrg} = $Self->{LayoutObject}->BuildSelection(
+                Data => \%NoteTypes,
+                Name => 'ArticleTypeID',
+                %ArticleType,
+            );
+            $Self->{LayoutObject}->Block(
+                Name => 'ArticleType',
+                Data => \%Param,
+            );
         }
-        $Param{ArticleTypeStrg} = $Self->{LayoutObject}->BuildSelection(
-            Data => \%NoteTypes,
-            Name => 'ArticleTypeID',
-            %ArticleType,
-        );
-        $Self->{LayoutObject}->Block(
-            Name => 'ArticleType',
-            Data => \%Param,
-        );
 
         # show time accounting box
         if ( $Self->{ConfigObject}->Get('Ticket::Frontend::AccountTime') ) {
@@ -1583,22 +1678,17 @@ sub _GetResponsible {
         Type  => 'Long',
         Valid => 1,
     );
-
-    # show all users
-    if ( $Self->{ConfigObject}->Get('Ticket::ChangeOwnerToEveryone') ) {
-        %ShownUsers = %AllGroupsMembers;
-    }
-
-    # show only users with responsible or rw pemissions in the queue
-    elsif ( $Param{QueueID} && !$Param{AllUsers} ) {
-        my $GID = $Self->{QueueObject}->GetQueueGroupID( QueueID => $Param{QueueID} );
+    if ( $Param{QueueID} && !$Param{AllUsers} ) {
+        my $GID = $Self->{QueueObject}->GetQueueGroupID(
+            QueueID => $Param{NewQueueID} || $Param{QueueID}
+        );
         my %MemberList = $Self->{GroupObject}->GroupMemberList(
             GroupID => $GID,
             Type    => 'responsible',
             Result  => 'HASH',
             Cached  => 1,
         );
-        for my $UserID ( keys %MemberList ) {
+        for my $UserID ( sort keys %MemberList ) {
             $ShownUsers{$UserID} = $AllGroupsMembers{$UserID};
         }
     }
@@ -1624,22 +1714,17 @@ sub _GetOwners {
         Type  => 'Long',
         Valid => 1,
     );
-
-    # show all users
-    if ( $Self->{ConfigObject}->Get('Ticket::ChangeOwnerToEveryone') ) {
-        %ShownUsers = %AllGroupsMembers;
-    }
-
-    # show only users with owner or rw pemissions in the queue
-    elsif ( $Param{QueueID} && !$Param{AllUsers} ) {
-        my $GID = $Self->{QueueObject}->GetQueueGroupID( QueueID => $Param{QueueID} );
+    if ( $Param{QueueID} && !$Param{AllUsers} ) {
+        my $GID = $Self->{QueueObject}->GetQueueGroupID(
+            QueueID => $Param{NewQueueID} || $Param{QueueID}
+        );
         my %MemberList = $Self->{GroupObject}->GroupMemberList(
             GroupID => $GID,
             Type    => 'owner',
             Result  => 'HASH',
             Cached  => 1,
         );
-        for my $UserID ( keys %MemberList ) {
+        for my $UserID ( sort keys %MemberList ) {
             $ShownUsers{$UserID} = $AllGroupsMembers{$UserID};
         }
     }
@@ -1752,7 +1837,7 @@ sub _GetFieldsToUpdate {
 
     my @UpdatableFields;
 
-    # set the fields that can be updatable via AJAXUpdate
+    # set the fields that can be updateable via AJAXUpdate
     if ( !$Param{OnlyDynamicFields} ) {
         @UpdatableFields
             = qw(
@@ -1761,7 +1846,7 @@ sub _GetFieldsToUpdate {
         );
     }
 
-    # cycle trough the activated Dynamic Fields for this screen
+    # cycle through the activated Dynamic Fields for this screen
     DYNAMICFIELD:
     for my $DynamicFieldConfig ( @{ $Self->{DynamicField} } ) {
         next DYNAMICFIELD if !IsHashRefWithData($DynamicFieldConfig);
