@@ -1,8 +1,6 @@
 # --
 # Kernel/System/PostMaster/Filter.pm - all functions to add/delete/list pm db filters
-# Copyright (C) 2001-2010 OTRS AG, http://otrs.org/
-# --
-# $Id: Filter.pm,v 1.24 2010-10-06 16:05:21 bes Exp $
+# Copyright (C) 2001-2013 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -14,8 +12,7 @@ package Kernel::System::PostMaster::Filter;
 use strict;
 use warnings;
 
-use vars qw(@ISA $VERSION);
-$VERSION = qw($Revision: 1.24 $) [1];
+use vars qw(@ISA);
 
 =head1 NAME
 
@@ -136,7 +133,7 @@ sub FilterAdd {
 
     for my $Type (qw(Match Set)) {
         my %Data = %{ $Param{$Type} };
-        for my $Key ( keys %Data ) {
+        for my $Key ( sort keys %Data ) {
             return if !$Self->{DBObject}->Do(
                 SQL => 'INSERT INTO postmaster_filter (f_name, f_stop, f_type, f_key, f_value)'
                     . ' VALUES (?, ?, ?, ?, ?)',
@@ -223,9 +220,3 @@ the enclosed file COPYING for license information (AGPL). If you
 did not receive this file, see L<http://www.gnu.org/licenses/agpl.txt>.
 
 =cut
-
-=head1 VERSION
-
-$Revision: 1.24 $ $Date: 2010-10-06 16:05:21 $
-
-=cut
