@@ -1,9 +1,7 @@
 #!/usr/bin/perl
 # --
 # bin/otrs.CleanUp.pl - to cleanup, remove used tmp data of ipc, database or fs
-# Copyright (C) 2001-2013 OTRS AG, http://otrs.org/
-# --
-# $Id: otrs.CleanUp.pl,v 1.10 2013/01/22 10:14:09 mg Exp $
+# Copyright (C) 2001-2013 OTRS AG, http://otrs.com/
 # --
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU AFFERO General Public License as published by
@@ -30,9 +28,6 @@ use lib dirname($RealBin);
 use lib dirname($RealBin) . '/Kernel/cpan-lib';
 use lib dirname($RealBin) . '/Custom';
 
-use vars qw($VERSION);
-$VERSION = qw($Revision: 1.10 $) [1];
-
 use Getopt::Std;
 use Kernel::Config;
 use Kernel::System::Encode;
@@ -46,8 +41,8 @@ use Kernel::System::AuthSession;
 my %Opts = ();
 getopt( 'h', \%Opts );
 if ( $Opts{'h'} ) {
-    print "otrs.CleanUp.pl <Revision $VERSION> - OTRS cleanup\n";
-    print "Copyright (C) 2001-2013 OTRS AG, http://otrs.org/\n";
+    print "otrs.CleanUp.pl - OTRS cleanup\n";
+    print "Copyright (C) 2001-2013 OTRS AG, http://otrs.com/\n";
     print "usage: otrs.CleanUp.pl \n";
     exit 1;
 }
